import numpy as np
import pandas as pd
import warnings as wa
import matplotlib.pyplot as plt

def split_xy(df, desiredfeatures, target):
    # Split data into target and desired features
    #
    # Takes in a dataframe and extracts the desired features used
    # to predict a specified target variable
    #
    # @param df A data frame.
    # @param desiredfeatures Target feature(s) you desire for your analysis in list or string form
    # @param target The target variable(s) you desire for your analysis in list or string form
    #
    # @return Two data frames or series each with number of columns equal to the number of variables passed in
    # as desiredfeatures and target. Whether it's a dataframe or pd.series depends on number of variables passed into
    # the arguments for desiredfeatures and target. If either passes in a list, it will return a dataframe.
    # If either passes in a single string, it will return a pd.series.
    # The dataframe/series (named X_df) contains only the columns listed in desiredfeatures from the input data frame.
    # The second dataframe/series (named y_df) contains only the columns listed in target from the input data frame.
    #
    # @examples
    # X_train, y_train = split_xy(train_df, ["feature1", "feature2", "feature3"], "G3")
    # X_train, y_train = split_xy(train_df, "G3", ["feature1", "feature2", "feature3"])
    
    X_df = None
    y_df = None
    
    if isinstance(desiredfeatures, str) and isinstance(target, str):
        if target == desiredfeatures:
            wa.warn("Desiredfeatures and target have features in common. Are you sure?", UserWarning)
    elif any(item in desiredfeatures for item in target) or any(item in target for item in desiredfeatures):
        list1 = set(desiredfeatures)
        both = list1.intersection(target)
        wa.warn("Desiredfeatures and target have features in common. Are you sure?", UserWarning)
    
    if isinstance(df, pd.DataFrame):
        if isinstance(desiredfeatures, str) or isinstance(desiredfeatures, list):
            X_df = df[desiredfeatures]
        else:
            raise TypeError("desiredfeatures is not a list or string")

        if isinstance(target, str) or isinstance(target, list):
            y_df = df[target]
        else:
            raise TypeError("target is not a list or string")
    else:
        raise TypeError("First argument is not a dataframe")
        
    return X_df, y_df


def plot_square_data(x_df, y_df, desiredFeatures, titles, txt):
    # Create plots to display in a square manner ( tiles of plots in 2x2, 3x3 or (n)by(n-1) configuration
    #
    # Takes in the required x and y dataframes that will be used to display the desired features in x_df 
    # alongside the data in y_df and the respective titles 
    #
    # @param x_df dataframe containing dependent variables
    # @param y_df dataframe containing independent variables
    # @param desiredFeatures Target feature(s) in x_df you desire for your analysis in list form
    # @param titles The respective titles(s) desired for your plot(s) in list form
    # @param txt the text to display as a string over the plot
    #
    # @return axy for testing purposes on the features of the plot
    # @return fig which is a copy of the plot able to be used in other functions
    #
    # @examples
    # plot_square_data(X_train, y_train, ["feature1", "feature2", "feature3"], ["title1", "title2", "title3], "This is Plot 1")
    # plot_square_data(dependent, independednts, ["income"], ["retirement_age"], "Income and Retirement Age Relation")
    

    if not (isinstance(x_df, pd.DataFrame) and isinstance(y_df, pd.Series) and x_df.shape[0] == y_df.shape[0]):
        raise TypeError("The first two arguments are not dataframes of equal length")
    if not (isinstance(desiredFeatures, list) and len(desiredFeatures) > 0 and isinstance(desiredFeatures[0], str)):
        raise TypeError("desiredFeatures is not a list of strings length at least 1")
    if not (isinstance(titles, list) and len(titles) > 0 and isinstance(titles[0], str) and len(titles) == len(desiredFeatures)):
        raise TypeError("titles is not a list of strings of length equal to desiredFeatures")
    if not isinstance(txt, str):
        raise TypeError("The last argument is not a string")


    # This will form a minimum sized box for the plots to go in, preferring horizontal boxes
    # will aways preffer to create a box rather than make vertical spanning blocks if there are many
    side_length_x = int(len(titles)**(1/2))
    side_length_y = int(len(titles)**(1/2))
    if side_length_x * side_length_y < len(titles):
        side_length_x += 1
        if side_length_x * side_length_y < len(titles):
            side_length_y += 1


    fig, axs = plt.subplots(side_length_y, side_length_x, figsize=(10,10))

    x = 0
    y = 0
    try:
        for i, feature in enumerate(desiredFeatures):
            #print(desiredFeatures[i])
            axs[y, x].scatter(x_df[desiredFeatures[i]], y_df)
            axs[y, x].set_title(titles[i])
            x += 1
            if x >= side_length_x:
                x = 0
                y += 1
    except:
        raise TypeError("desiredFeature is not in dependent dataframe")


    # plt.figtext(0.5, 0.05, txt, wrap=True, horizontalalignment='center', fontsize=12)
    fig1 = plt.gcf()
    # plt.show()

    return axs,fig1

def list_abs(pre, transformers, steps, features):
    # This is the abstract function for list, used in Results part of the notebook
    # @param pre is the preprocessor we're working with
    # @param transformers is the name of the transformers for pre
    # @param steps is the steps name for pre
    # @param features is the category of features we're focusing on
    #
    # @return a list containing all the features listed under the category (passed in as input "features")
    #
    # @examples
    # list_abs(preprocessor, "pipeline-2", "onehotencoder", categorical_features)
    
    #See if transformers and steps are Strings
    if (not isinstance(transformers, str)) or (not isinstance(steps, str)):
        raise TypeError("Transformers and Steps need to be strings")
        
    #See if features is a list of Strings
    if (not isinstance(features, list)):
        raise TypeError("features need to be a list")
        
    #If both tests are passed: we generate the outcome and return
    else:
        ret = list(
        pre.named_transformers_[transformers]
        .named_steps[steps]
        .get_feature_names_out(features)
        )
    
    return ret


# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['grouponefunctions']

package_data = \
{'': ['*']}

install_requires = \
['matplotlib>=3.5.1', 'numpy==1.21.5', 'pandas>=1.4.2']

setup_kwargs = {
    'name': 'grouponefunctions',
    'version': '0.1.8',
    'description': 'A package containing the necessary functions to smoothly run the analysis in DSCI-310-Group-1',
    'long_description': '# grouponefunctions\n\nA package containing the necessary functions to smoothly run the analysis in DSCI-310-Group-1.\n\n## Installation\n\nNote: The installation is not likely to proceed for Python version lower than 3.9.10. Please check your Python version the code below.\n\n```bash\npython --version\n```\n\nThen you can install the package by the command below.\n\n```bash\npip install grouponefunctions\n```\n\n## Usage\n\n`grouponefunctions` is primarily used for analysis of [Predicting students’ grades using multi-variable regression](https://github.com/DSCI-310/DSCI-310-Group-1), but contains three general use functions. To implement the functions, `import` our package `grouponefunctions` as a module. The package has three functions: `splitxy`, `listfun`, and `plotsquaredata`. \n\n### `split_xy`\n\n `split_xy` can be used to split data into predictor feature and target variables as follows:\n\n```python\nfrom grouponefunctions.grouponefunctions import split_xy\n\nX_train, y_train = split_xy(train_df, ["feature1", "feature2", "feature3"], target)\nX_test, y_test = split_xy(test_df, ["feature1", "feature2", "feature3"], target) \n```\n\n### `plot_square_data`\n\nThis function is capable at creating plots to display in a grid configuration.\n\n```python\nfrom grouponefunctions.grouponefunction import plot_square_data\n\nplot_square_data(X_train, y_train, ["feature1", "feature2", "feature3"], ["title1", "title2", "title3], "This is Plot 1")\nplot_square_data(dependent, independednts, ["income"], ["retirement_age"], "Income and Retirement Age Relation")\n```\n\n### `list_abs`\n\nThis function is the abstract function for list, used in Results part in the analysis. \n\n```python\nfrom grouponefunctions.grouponefunction import list_abs\n\nlist_abs(preprocessor, "pipeline-2", "onehotencoder", categorical_features)\n```\n\n\n\n## Contributing\n\nInterested in contributing? Check out the contributing guidelines. Please note that this project is released with a Code of Conduct. By contributing to this project, you agree to abide by its terms.\n\n## License\n\n`grouponefunctions` was created by Andres Zepeda Perez, Daniel Hou, Zizhen Guo, Timothy Zhou. It is licensed under the terms of the MIT license.\n\n## Credits\n\n`grouponefunctions` was created with [`cookiecutter`](https://cookiecutter.readthedocs.io/en/latest/) and the `py-pkgs-cookiecutter` [template](https://github.com/py-pkgs/py-pkgs-cookiecutter).\n',
    'author': 'Andres Perez, Daniel Hou, Zizhen Guo, Timothy Zhou',
    'author_email': None,
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9,<3.11',
}


setup(**setup_kwargs)
